<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities\Query;

class ParaGeraisExpImpostoService {

    // exporta lista de produtos
    const EXPORT_ALLPROD  = "E_ALLPROD";
    // exporta todos produtos
    const EXPORT_LISTPROD = "E_LISTPROD";
    // todos os produtos 
    const T_PRODUCT = "T";
    // exportação concluída
    const SUCCESS_PC = "SUCCESS";
    // erro na exportação
    const ERROR_PC = "ERROR";
    
    public function __construct(EntityManager $entityManager)  {
        $this->entityManager = $entityManager;
    }
  
    public function exportImpost($dataExport){
        // seleciona qual tipo de exportação será realizada
        $status = null;
        $data   = $dataExport['DATA'];
        
        try {
            self::controlTransaction('begin');            
        } catch (Exception $e) {
            return self::ERROR_PC;
        }
        
        switch($dataExport['TYPE']) {
            case "N":
                $status = self::exportNivel($data);
                break;

            case "L":
                $status = self::exportListProd($data);
                break;
            
            case "I":
                $status = self::exportInterval($data);
                break;
        }

        return $status;
    }

    public function controlTransaction($type){
    // controla transação com banco    
        try {
            switch ($type) {
                case 'begin':
                // inicia transação sem auto-commit
                    $this->entityManager->getConnection()->beginTransaction();
                    break;
                case 'commit':
                // salva transação
                    $this->entityManager->getConnection()->commit();
                    break;
                case 'rollback':
                // descarta transação
                    $this->entityManager->getConnection()->rollBack();
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportAllProd($data){
        $paramsQuery  = Array();

        try {
            // busca PRODUTOS da tabela de origem
            self::formatGetProdOri($paramsQuery, $data);
            $data['CDPRODUTOORI'] = $this->entityManager->getConnection()->fetchAll(self::GET_ALL_ORIGIN_PRODUCTS, $paramsQuery);
            foreach($data['CDFILIALDES'] as $destino){
                // deleta PRODUTOS EM COMUM na tabela de destino
                foreach($data['CDPRODUTOORI'] as $produto) {
                    self::formatFiliDes($paramsQuery,$data, $destino, $produto);
                    $this->entityManager->getConnection()->executeQuery(self::DELETE_DESTINY_ALIQIMPFIS, $paramsQuery);
                }
                // insere os dados da tabela origem na tabela destino
                self::formatFiliDesOri($paramsQuery,$data, $destino);
                $this->entityManager->getConnection()->executeQuery(self::INSERT_ORIGIN_ALLALIQIMPFIS, $paramsQuery);
            }
            self::controlTransaction('commit');
            return self::SUCCESS_PC;
        } catch (Exception $e) {
            self::controlTransaction('rollback');
            return self::ERROR_PC;
        }
    }

    public function exportListProd($data){
        $paramsQuery = Array();

        try {
            foreach($data['CDARVPROD'] as $PRODUTO) {
                self::formatFiliDesProd($paramsQuery, $data, $PRODUTO);
                $this->entityManager->getConnection()->executeQuery(self::DELETE_DESTINY_LIST_PROD, $paramsQuery); 
                
                // deleta o produto da tabela destino
                self::formatFiliDesProdInsert($paramsQuery, $data, $PRODUTO);
                
                foreach($data['CDFILIALDES'] as $destino){
                   $paramsQuery['CDFILIALDES'] = $destino;
                   // insere o produto da tabela origem na tabela destino
                   $this->entityManager->getConnection()->executeQuery(self::INSERT_DESTINY_LIST_PROD, $paramsQuery);
               }
            }
            
            self::controlTransaction('commit');
            return self::SUCCESS_PC;
        } catch (Exception $e) {
            self::controlTransaction('rollback');
            return self::ERROR_PC;
        }
    }
    
    public function exportNivel($data) {
        $params = array(
            ':CDFILIAL' => $data['CDFILIALORI'],
            ':NIVEL'    => $data['NIVEL'].'%'
        );
        
        try {
            // pega todos os produtos que tem o nivel escolhido
            $produtos = $this->entityManager->getConnection()->fetchAll(self::GET_PROD_NIVEL, $params);
            
            if(count($produtos)) {
                foreach($data['CDFILIALDES'] as $destino) {
                    foreach($produtos as $produto) {
                        $paramsDelete = array(
                            ':CDFILIALDES' => $destino,
                            ':CDPRODUTO'   => $produto['CDPRODUTO'],
                            ':NRORG'       => $data['NRORG']
                        );
                        
                        // deleta o produto da unidade destino
                        $this->entityManager->getConnection()->executeQuery(self::DELETE_DESTINY_LIST_PROD, $paramsDelete);
                    
                        $paramsInsert = array(
                            ':CDFILIALDES' => $destino,
                            ':CDFILIALORI' => $data['CDFILIALORI'],
                            ':CDPRODUTO'   => $produto['CDPRODUTO'],
                            ':NRORG'       => $data['NRORG']
                        );
                        
                        // insere o produto da unidade destino
                        $this->entityManager->getConnection()->executeQuery(self::INSERT_DESTINY_LIST_PROD, $paramsInsert);
                    }
                }
            }
            
            self::controlTransaction('commit');
            return self::SUCCESS_PC;
        } catch (Exception $e) {
            self::controlTransaction('rollback');
            return self::ERROR_PC;
        }
    }
    
    public function exportInterval($data) {
        $params = array(
            ':CDFILIAL'      => $data['CDFILIALORI'],
            ':CDPRODINICIAL' => $data['CDPRODINICIAL'],
            ':CDPRODFINAL'   => $data['CDPRODFINAL']
        );
        
        try {
            // pega o intervalo de produtos
            $produtos = $this->entityManager->getConnection()->fetchAll(self::GET_PROD_INTERVALO, $params);
            
            if(count($produtos)) {
                foreach($data['CDFILIALDES'] as $destino) {
                    foreach($produtos as $produto) {
                        $paramsDelete = array(
                            ':CDFILIALDES' => $destino,
                            ':CDPRODUTO'   => $produto['CDPRODUTO'],
                            ':NRORG'       => $data['NRORG']
                        );
                        
                        // deleta o produto da unidade destino
                        $this->entityManager->getConnection()->executeQuery(self::DELETE_DESTINY_LIST_PROD, $paramsDelete);
                    
                        $paramsInsert = array(
                            ':CDFILIALDES' => $destino,
                            ':CDFILIALORI' => $data['CDFILIALORI'],
                            ':CDPRODUTO'   => $produto['CDPRODUTO'],
                            ':NRORG'       => $data['NRORG']
                        );
                        
                        // insere o produto da unidade destino
                        $this->entityManager->getConnection()->executeQuery(self::INSERT_DESTINY_LIST_PROD, $paramsInsert);
                    }
                }
            }
            
            self::controlTransaction('commit');
            return self::SUCCESS_PC;
        } catch (Exception $e) {
            self::controlTransaction('rollback');
            return self::ERROR_PC;
        }
    }
    
    public function formatGetProdOri(&$paramsQuery, $data){
        $paramsQuery = array(
          'CDFILIALORI' => $data['CDFILIALORI'],
          'NRORG'       => $data['NRORG']
        );
    }

    public function formatFiliDesOri(&$paramsQuery, $data, $destino){
        $paramsQuery = array(
          'CDFILIALDES' => $destino,
          'CDFILIALORI' => $data['CDFILIALORI'],
          'NRORG'    => $data['NRORG']
        );
    }
    
    public function formatFiliDes(&$paramsQuery, $data, $destino, $produto){
        $paramsQuery = array(
          'CDFILIALDES' => $destino,
          'CDPRODUTOORI'=> $produto['CDPRODUTO'],
          'NRORG'       => $data['NRORG']
        );
    }
    
    public function formatFiliDesProd(&$paramsQuery, $data, $PRODUTO){
        $paramsQuery = array(
          'CDFILIALDES'=> implode('_', $data['CDFILIALDES']),
          'NRORG'      => $data['NRORG'],
          'CDPRODUTO'  => $PRODUTO
        );  
    }
    
    public function formatFiliDesProdInsert(&$paramsQuery, $data, $PRODUTO){
        $paramsQuery = array(
          'CDFILIALDES'=> $data['CDFILIALDES'],
          'NRORG'      => $data['NRORG'],
          'CDFILIALORI' => $data['CDFILIALORI'],
          'CDPRODUTO'  => $PRODUTO
        );        
    }
    
    const GET_ALL_ORIGIN_PRODUCTS = "
    	SELECT CDPRODUTO
		  FROM ALIQIMPFIS
		 WHERE NRORG = :NRORG
		   AND CDFILIAL = :CDFILIALORI
    ";

    const DELETE_DESTINY_ALIQIMPFIS = "
      DELETE FROM ALIQIMPFIS WHERE CDFILIAL = :CDFILIALDES AND CDPRODUTO = :CDPRODUTOORI AND NRORG = :NRORG 
    ";

    const INSERT_ORIGIN_ALLALIQIMPFIS = "
      INSERT INTO ALIQIMPFIS
        ( CDFILIAL, CDPRODUTO, VRPEALIMPFIS, CDIMPOSTO, IDTPIMPOSFIS,
          IDINCIDEPISCOF, VRALIQIBPT, VRALIQIBPTES, CDCSTICMS, CDCSTPISCOF, 
          CDCFOPPFIS, VRALIQPIS, VRALIQCOFINS, CDREGRAIT, VRPERCREDUCAO, IDMODALBASECALC, 
          CDREGRAITPC, CDCEST, NRORG, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES,VRALIQISS,CDSERVICOISS,CDITEMLISTASERV,IDEXIGIBILISS,IDINCENTIVOISS
        )
      SELECT :CDFILIALDES, CDPRODUTO, VRPEALIMPFIS, CDIMPOSTO, IDTPIMPOSFIS, 
             IDINCIDEPISCOF, VRALIQIBPT, VRALIQIBPTES, CDCSTICMS, CDCSTPISCOF,
             CDCFOPPFIS, VRALIQPIS, VRALIQCOFINS, CDREGRAIT, VRPERCREDUCAO, IDMODALBASECALC, 
             CDREGRAITPC, CDCEST, NRORG, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES,VRALIQISS,CDSERVICOISS,CDITEMLISTASERV,IDEXIGIBILISS,IDINCENTIVOISS
        FROM ALIQIMPFIS
       WHERE (CDFILIAL = :CDFILIALORI)
         AND NRORG = :NRORG
    ";
      
    const DELETE_DESTINY_LIST_PROD = "
      DELETE
        FROM ALIQIMPFIS
       WHERE (INSTR(:CDFILIALDES, CDFILIAL) <> 0)
         AND (INSTR(:CDPRODUTO, CDPRODUTO) <> 0)
         AND NRORG = :NRORG
    ";
      
    const INSERT_DESTINY_LIST_PROD = "
      INSERT INTO ALIQIMPFIS
        (
          CDFILIAL, CDPRODUTO, VRPEALIMPFIS, CDIMPOSTO, IDTPIMPOSFIS,
          IDINCIDEPISCOF, VRALIQIBPT, VRALIQIBPTES, CDCSTICMS, CDCSTPISCOF, 
          CDCFOPPFIS, VRALIQPIS, VRALIQCOFINS, CDREGRAIT, VRPERCREDUCAO, IDMODALBASECALC, 
          CDREGRAITPC, CDCEST, NRORG, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES,VRALIQISS,CDSERVICOISS,CDITEMLISTASERV,IDEXIGIBILISS,IDINCENTIVOISS
        )
      SELECT :CDFILIALDES, CDPRODUTO, VRPEALIMPFIS, CDIMPOSTO, IDTPIMPOSFIS, 
             IDINCIDEPISCOF, VRALIQIBPT, VRALIQIBPTES, CDCSTICMS, CDCSTPISCOF,
             CDCFOPPFIS, VRALIQPIS, VRALIQCOFINS, CDREGRAIT, VRPERCREDUCAO, IDMODALBASECALC, 
             CDREGRAITPC, CDCEST, NRORG, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES,VRALIQISS,CDSERVICOISS,CDITEMLISTASERV,IDEXIGIBILISS,IDINCENTIVOISS
        FROM ALIQIMPFIS
       WHERE (CDFILIAL = :CDFILIALORI)
         AND (INSTR(:CDPRODUTO, CDPRODUTO) <> 0)
         AND NRORG = :NRORG
    ";
    
    const GET_PROD_NIVEL = "
        SELECT A.CDPRODUTO
            FROM ALIQIMPFIS A
                JOIN PRODUTO P
                    ON A.CDPRODUTO = P.CDPRODUTO
            WHERE A.CDFILIAL = :CDFILIAL
                AND P.CDARVPROD LIKE :NIVEL
    ";
    
    const GET_PROD_INTERVALO = "
        SELECT P.CDPRODUTO
            FROM ALIQIMPFIS A
                JOIN PRODUTO P
                    ON A.CDPRODUTO = P.CDPRODUTO
            WHERE A.CDFILIAL = :CDFILIAL
                AND P.CDPRODUTO BETWEEN :CDPRODINICIAL AND :CDPRODFINAL
    ";
}